@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,10 +7,11 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: details.c 7463 2008-12-22 05:39:02Z charles $
+ * $Id: details.c 7722 2009-01-16 17:31:10Z charles $
  */
 
 #include <errno.h>
+#include <math.h> /* ceil() */
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -1010,8 +1011,10 @@ refresh_activity( GtkWidget * top )
     char            sizeStr2[64];
     char            buf[128];
     const tr_stat * stat = tr_torrent_stat( a->gtor );
+    const tr_info * info = tr_torrent_info( a->gtor );
     const double    complete = stat->percentComplete * 100.0;
     const double    done = stat->percentDone * 100.0;
+    const double    verifiedPieceCount = (double)stat->haveValid / info->pieceSize;
 
     pch = tr_torrent_status_str( a->gtor );
     gtk_label_set_text ( GTK_LABEL( a->state_lb ), pch );
@@ -1034,9 +1037,10 @@ refresh_activity( GtkWidget * top )
     tr_strlsize( sizeStr2, stat->haveValid,
                 sizeof( sizeStr2 ) );
     /* %1$s is total size of what we've saved to disk
-       %2$s is how much of it's passed the checksum test */
-    g_snprintf( buf, sizeof( buf ), _(
-                    "%1$s (%2$s verified)" ), sizeStr, sizeStr2 );
+       %2$s is how much of it's passed the checksum test
+       %3$s is how many pieces are verified */
+    g_snprintf( buf, sizeof( buf ), _( "%1$s (%2$s verified in %3$d pieces)" ),
+                sizeStr, sizeStr2, (int)ceil(verifiedPieceCount) );
     gtk_label_set_text( GTK_LABEL( a->have_lb ), buf );
 
     tr_strlsize( sizeStr, stat->downloadedEver, sizeof( sizeStr ) );
@@ -1478,16 +1482,11 @@ torrent_inspector_new( GtkWindow * parent,
     guint           tag;
     GtkWidget *     d, *n, *w;
     tr_torrent *    tor = tr_torrent_handle ( gtor );
-    char            sizeStr[64];
     char            title[512];
     const tr_info * info = tr_torrent_info ( gtor );
 
     /* create the dialog */
-    tr_strlsize( sizeStr, info->totalSize, sizeof( sizeStr ) );
-    /* %1$s is torrent name
-       %2$s its file size */
-    g_snprintf( title, sizeof( title ), _(
-                    "Details for %1$s (%2$s)" ), info->name, sizeStr );
+    g_snprintf( title, sizeof( title ), _( "%s Properties" ), info->name );
     d = gtk_dialog_new_with_buttons ( title, parent, 0,
                                       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                       NULL );
