@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: notify.c 6795 2008-09-23 19:11:04Z charles $
+ * $Id: notify.c 7722 2009-01-16 17:31:10Z charles $
  */
 
 #include <string.h>
@@ -15,7 +15,9 @@
  #include <gio/gio.h>
 #endif
 #include <glib/gi18n.h>
+#include "conf.h"
 #include "notify.h"
+#include "tr-prefs.h"
 #include "util.h"
 
 #ifndef HAVE_LIBNOTIFY
@@ -60,20 +62,27 @@ notifyCallback( NotifyNotification * n UNUSED,
 void
 tr_notify_send( TrTorrent *tor )
 {
-    const tr_info *      info = tr_torrent_info( tor );
-    NotifyNotification * n = notify_notification_new( _(
-                                                          "Torrent Complete" ),
-                                                      info->name,
-                                                      "transmission", NULL );
+    if( pref_flag_get( PREF_KEY_SHOW_DESKTOP_NOTIFICATION ) )
+    {
+        const tr_info * info = tr_torrent_info( tor );
+        NotifyNotification * n;
 
-    if( info->fileCount == 1 )
-        notify_notification_add_action( n, "file", _( "Open File" ),
-                                        NOTIFY_ACTION_CALLBACK(
-                                            notifyCallback ), tor, NULL );
-    notify_notification_add_action( n, "folder", _( "Open Folder" ),
-                                    NOTIFY_ACTION_CALLBACK(
-                                        notifyCallback ), tor, NULL );
-    notify_notification_show( n, NULL );
+        n = notify_notification_new( _( "Torrent Complete" ),
+                                     info->name,
+                                     "transmission", NULL );
+
+        if( info->fileCount == 1 )
+            notify_notification_add_action(
+                n, "file", _( "Open File" ),
+                NOTIFY_ACTION_CALLBACK( notifyCallback ), tor,
+                NULL );
+
+        notify_notification_add_action(
+            n, "folder", _( "Open Folder" ),
+            NOTIFY_ACTION_CALLBACK( notifyCallback ), tor, NULL );
+
+        notify_notification_show( n, NULL );
+    }
 }
 
 #endif
