@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-core.c 7463 2008-12-22 05:39:02Z charles $
+ * $Id: tr-core.c 7861 2009-02-09 20:11:32Z charles $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -163,7 +163,6 @@ tr_core_dispose( GObject * obj )
     {
         GObjectClass * parent;
 
-        pref_save( );
         core->priv = NULL;
 
         parent = g_type_class_peek( g_type_parent( TR_CORE_TYPE ) );
@@ -449,11 +448,11 @@ tr_core_apply_defaults( tr_ctor * ctor )
 
     if( tr_ctorGetPeerLimit( ctor, TR_FORCE, NULL ) )
         tr_ctorSetPeerLimit( ctor, TR_FORCE,
-                            pref_int_get( PREF_KEY_MAX_PEERS_PER_TORRENT ) );
+                             pref_int_get( TR_PREFS_KEY_PEER_LIMIT_TORRENT ) );
 
     if( tr_ctorGetDownloadDir( ctor, TR_FORCE, NULL ) )
     {
-        const char * path = pref_string_get( PREF_KEY_DOWNLOAD_DIR );
+        const char * path = pref_string_get( TR_PREFS_KEY_DOWNLOAD_DIR );
         tr_ctorSetDownloadDir( ctor, TR_FORCE, path );
     }
 }
@@ -583,11 +582,16 @@ prefsChanged( TrCore *      core,
         gboolean     isReversed = pref_flag_get( PREF_KEY_SORT_REVERSED );
         setSort( core, mode, isReversed );
     }
-    else if( !strcmp( key, PREF_KEY_MAX_PEERS_GLOBAL ) )
+    else if( !strcmp( key, TR_PREFS_KEY_PEER_LIMIT_GLOBAL ) )
     {
         const uint16_t val = pref_int_get( key );
         tr_sessionSetPeerLimit( tr_core_session( core ), val );
     }
+    else if( !strcmp( key, TR_PREFS_KEY_PEER_LIMIT_TORRENT ) )
+    {
+        const uint16_t val = pref_int_get( key );
+        tr_sessionSetPeerLimitPerTorrent( tr_core_session( core ), val );
+    }
     else if( !strcmp( key, PREF_KEY_INHIBIT_HIBERNATION ) )
     {
         maybeInhibitHibernation( core );
@@ -683,10 +687,9 @@ tr_core_new( tr_session * session )
     prefsChanged( core, PREF_KEY_SORT_MODE, NULL );
     prefsChanged( core, PREF_KEY_SORT_REVERSED, NULL );
     prefsChanged( core, PREF_KEY_DIR_WATCH_ENABLED, NULL );
-    prefsChanged( core, PREF_KEY_MAX_PEERS_GLOBAL, NULL );
+    prefsChanged( core, TR_PREFS_KEY_PEER_LIMIT_GLOBAL, NULL );
     prefsChanged( core, PREF_KEY_INHIBIT_HIBERNATION, NULL );
-    g_signal_connect( core, "prefs-changed", G_CALLBACK(
-                          prefsChanged ), NULL );
+    g_signal_connect( core, "prefs-changed", G_CALLBACK( prefsChanged ), NULL );
 
     return core;
 }
@@ -699,6 +702,7 @@ tr_core_close( TrCore * core )
     if( session )
     {
         core->priv->session = NULL;
+        pref_save( session );
         tr_sessionClose( session );
     }
 }
@@ -818,7 +822,7 @@ tr_core_load( TrCore * self,
     if( forcePaused )
         tr_ctorSetPaused( ctor, TR_FORCE, TRUE );
     tr_ctorSetPeerLimit( ctor, TR_FALLBACK,
-                        pref_int_get( PREF_KEY_MAX_PEERS_PER_TORRENT ) );
+                         pref_int_get( TR_PREFS_KEY_PEER_LIMIT_TORRENT ) );
 
     torrents = tr_sessionLoadTorrents ( tr_core_session( self ), ctor, &count );
     for( i = 0; i < count; ++i )
@@ -857,37 +861,50 @@ add_filename( TrCore *     core,
 
     if( filename && session )
     {
-        int       err;
-        tr_ctor * ctor = tr_ctorNew( session );
+        tr_ctor * ctor;
+
+        ctor = tr_ctorNew( session );
         tr_core_apply_defaults( ctor );
         tr_ctorSetPaused( ctor, TR_FORCE, !doStart );
+
         if( tr_ctorSetMetainfoFromFile( ctor, filename ) )
         {
             tr_core_errsig( core, TR_EINVALID, filename );
             tr_ctorFree( ctor );
         }
-        else if( ( err = tr_torrentParse( session, ctor, NULL ) ) )
-        {
-            /* don't complain about .torrent files in the watch directory
-               that have already been added... that gets annoying, and we
-               don't want to nag about cleaning up the watch dir */
-            const gboolean quiet = ( err == TR_EDUPLICATE )
-                                && ( core->priv->adding_from_watch_dir );
-            if( !quiet )
-                tr_core_errsig( core, err, filename );
-
-            tr_ctorFree( ctor );
-        }
-        else if( doPrompt )
-            g_signal_emit( core, TR_CORE_GET_CLASS(
-                               core )->promptsig, 0, ctor );
         else
         {
-            tr_torrent * tor = tr_torrentNew( session, ctor, &err );
-            if( err )
-                tr_core_errsig( core, err, filename );
-            else
-                tr_core_add_torrent( core, tr_torrent_new_preexisting( tor ) );
+            tr_info inf;
+            int err = tr_torrentParse( session, ctor, &inf );
+
+            switch( err )
+            {
+                case TR_EINVALID:
+                    tr_core_errsig( core, err, filename );
+                    break;
+
+                case TR_EDUPLICATE:
+                    /* don't complain about .torrent files in the watch directory
+                     * that have already been added... that gets annoying and we
+                     * don't want to be naggign users to clean up their watch dirs */
+                    if( !core->priv->adding_from_watch_dir )
+                        tr_core_errsig( core, err, inf.name );
+                    tr_metainfoFree( &inf );
+                    break;
+
+                default:
+                    if( doPrompt )
+                        g_signal_emit( core, TR_CORE_GET_CLASS( core )->promptsig, 0, ctor );
+                    else {
+                        TrTorrent * gtor = tr_torrent_new_ctor( session, ctor, &err );
+                        if( err )
+                            tr_core_errsig( core, err, filename );
+                        else
+                            tr_core_add_torrent( core, gtor );
+                    }
+                    tr_metainfoFree( &inf );
+                    break;
+            }
         }
     }
 }
@@ -899,8 +916,8 @@ tr_core_add_file( TrCore *          core,
                   GError     ** err UNUSED )
 {
     add_filename( core, filename,
-                 pref_flag_get( PREF_KEY_START ),
-                 pref_flag_get( PREF_KEY_OPTIONS_PROMPT ) );
+                  pref_flag_get( PREF_KEY_START ),
+                  pref_flag_get( PREF_KEY_OPTIONS_PROMPT ) );
     *success = TRUE;
     return TRUE;
 }
@@ -1190,13 +1207,9 @@ maybeInhibitHibernation( TrCore * core )
 
     /* always allow hibernation when all the torrents are paused */
     if( inhibit ) {
-        gboolean active = FALSE;
-        tr_session *  session = tr_core_session( core );
-        tr_torrent * tor = NULL;
-        while(( tor = tr_torrentNext( session, tor )))
-            if(( active = ( tr_torrentGetActivity( tor ) != TR_STATUS_STOPPED )))
-                break;
-        if( !active )
+        tr_session * session = tr_core_session( core );
+
+        if( tr_sessionGetActiveTorrentCount( session ) == 0 )
             inhibit = FALSE;
     }
 
@@ -1212,7 +1225,7 @@ commitPrefsChange( TrCore *     core,
                    const char * key )
 {
     g_signal_emit( core, TR_CORE_GET_CLASS( core )->prefsig, 0, key );
-    pref_save( );
+    pref_save( tr_core_session( core ) );
 }
 
 void
