@@ -5,7 +5,7 @@ static const char * fallback_ui_file =
     "      <menuitem action='add-torrent-menu'/>\n"
     "      <menuitem action='new-torrent'/>\n"
     "      <separator/>\n"
-    "      <menuitem action='show-torrent-details'/>\n"
+    "      <menuitem action='show-torrent-properties'/>\n"
     "      <menuitem action='open-torrent-folder'/>\n"
     "      <separator/>\n"
     "      <menuitem action='start-torrent'/>\n"
@@ -55,11 +55,11 @@ static const char * fallback_ui_file =
     "    <toolitem action='pause-torrent'/>\n"
     "    <toolitem action='remove-torrent'/>\n"
     "    <separator/>\n"
-    "    <toolitem action='show-torrent-details'/>\n"
+    "    <toolitem action='show-torrent-properties'/>\n"
     "  </toolbar>\n"
     "\n"
     "  <popup name='main-window-popup'>\n"
-    "    <menuitem action='show-torrent-details'/>\n"
+    "    <menuitem action='show-torrent-properties'/>\n"
     "    <menuitem action='open-torrent-folder'/>\n"
     "    <separator/>\n"
     "    <menuitem action='start-torrent'/>\n"
@@ -78,6 +78,8 @@ static const char * fallback_ui_file =
     "    <menuitem action='toggle-message-log'/>\n"
     "    <menuitem action='show-about-dialog'/>\n"
     "    <separator/>\n"
+    "    <menuitem action='pause-all-torrents'/>\n"
+    "    <separator/>\n"
     "    <menuitem action='quit'/>\n"
     "  </popup>\n"
     "\n"
