@@ -2,28 +2,39 @@
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
+
+#include "event.h"
+
 #include "transmission.h"
 #include "bencode.h"
 #include "json.h"
 #include "utils.h" /* tr_free */
 
-#define VERBOSE 0
+#undef VERBOSE
 
 static int test = 0;
 
-#define check( A ) \
+#ifdef VERBOSE
+  #define check( A ) \
     { \
         ++test; \
         if( A ){ \
-            if( VERBOSE ) \
-                fprintf( stderr, "PASS test #%d (%s, %d)\n", test, __FILE__,\
-                         __LINE__ );\
+            fprintf( stderr, "PASS test #%d (%s, %d)\n", test, __FILE__, __LINE__ ); \
         } else { \
-            fprintf( stderr, "FAIL test #%d (%s, %d)\n", test, __FILE__,\
-                     __LINE__ ); \
+            fprintf( stderr, "FAIL test #%d (%s, %d)\n", test, __FILE__, __LINE__ ); \
             return test; \
         } \
     }
+#else
+  #define check( A ) \
+    { \
+        ++test; \
+        if( !( A ) ){ \
+            fprintf( stderr, "FAIL test #%d (%s, %d)\n", test, __FILE__, __LINE__ ); \
+            return test; \
+        } \
+    }
+#endif
 
 static int
 testInt( void )
@@ -309,10 +320,11 @@ testJSONSnippet( const char * benc_str,
                  const char * expected )
 {
     tr_benc top;
-    char *  serialized;
+    struct evbuffer * buf = tr_getBuffer( );
+    char * serialized;
 
     tr_bencLoad( benc_str, strlen( benc_str ), &top, NULL );
-    serialized = tr_bencSaveAsJSON( &top, NULL );
+    serialized = tr_bencSaveAsJSON( &top, buf );
     stripWhitespace( serialized );
 #if 0
     fprintf( stderr, "benc: %s\n", benc_str );
@@ -320,8 +332,8 @@ testJSONSnippet( const char * benc_str,
     fprintf( stderr, "want: %s\n", expected );
 #endif
     check( !strcmp( serialized, expected ) );
-    tr_free( serialized );
     tr_bencFree( &top );
+    tr_releaseBuffer( buf );
     return 0;
 }
 
@@ -361,6 +373,55 @@ testJSON( void )
     return 0;
 }
 
+static int 
+testMerge( void )
+{ 
+    tr_benc dest, src; 
+    int64_t i; 
+    const char * s; 
+
+    /* initial dictionary (default values)  */
+    tr_bencInitDict( &dest, 10 ); 
+    tr_bencDictAddInt( &dest, "i1", 1 ); 
+    tr_bencDictAddInt( &dest, "i2", 2 ); 
+    tr_bencDictAddInt( &dest, "i4", -35 ); /* remains untouched */
+    tr_bencDictAddStr( &dest, "s5", "abc" ); 
+    tr_bencDictAddStr( &dest, "s6", "def" ); 
+    tr_bencDictAddStr( &dest, "s7", "127.0.0.1" ); /* remains untouched */
+
+    /* new dictionary, will overwrite items in dest  */
+    tr_bencInitDict( &src, 10 ); 
+    tr_bencDictAddInt( &src, "i1", 1 );     /* same value */
+    tr_bencDictAddInt( &src, "i2", 4 );     /* new value */
+    tr_bencDictAddInt( &src, "i3", 3 );     /* new key:value */
+    tr_bencDictAddStr( &src, "s5", "abc" ); /* same value */
+    tr_bencDictAddStr( &src, "s6", "xyz" ); /* new value */
+    tr_bencDictAddStr( &src, "s8", "ghi" ); /* new key:value */
+
+    tr_bencMergeDicts( &dest, /*const*/ &src ); 
+
+    check( tr_bencDictFindInt( &dest, "i1", &i )); 
+    check( i == 1); 
+    check( tr_bencDictFindInt( &dest, "i2", &i )); 
+    check( i == 4); 
+    check( tr_bencDictFindInt( &dest, "i3", &i )); 
+    check( i == 3); 
+    check( tr_bencDictFindInt( &dest, "i4", &i )); 
+    check( i == -35); 
+    check( tr_bencDictFindStr( &dest, "s5", &s )); 
+    check( strcmp( "abc", s ) == 0 ); 
+    check( tr_bencDictFindStr( &dest, "s6", &s )); 
+    check( strcmp( "xyz", s ) == 0 ); 
+    check( tr_bencDictFindStr( &dest, "s7", &s )); 
+    check( strcmp( "127.0.0.1", s ) == 0 ); 
+    check( tr_bencDictFindStr( &dest, "s8", &s )); 
+    check( strcmp( "ghi", s ) == 0 ); 
+
+    tr_bencFree( &dest ); 
+    tr_bencFree( &src ); 
+    return 0; 
+} 
+
 static int
 testStackSmash( int depth )
 {
@@ -408,6 +469,9 @@ main( void )
     if(( i = testJSON( )))
         return i;
 
+    if(( i = testMerge( )))
+        return i;
+
 #ifndef WIN32
     i = testStackSmash( 1000000 );
 #else
