@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: handshake.c 7455 2008-12-22 00:51:14Z charles $
+ * $Id: handshake.c 7884 2009-02-12 20:43:09Z charles $
  */
 
 #include <assert.h>
@@ -20,6 +20,7 @@
 #include <event.h>
 
 #include "transmission.h"
+#include "session.h"
 #include "bencode.h"
 #include "clients.h"
 #include "crypto.h"
@@ -58,7 +59,10 @@ enum
     VC_LENGTH                      = 8,
     KEY_LEN                        = 96,
     CRYPTO_PROVIDE_PLAINTEXT       = 1,
-    CRYPTO_PROVIDE_CRYPTO          = 2
+    CRYPTO_PROVIDE_CRYPTO          = 2,
+
+    /* how long to wait before giving up on a handshake */
+    HANDSHAKE_TIMEOUT_MSEC         = 60 * 1000
 };
 
 
@@ -105,6 +109,7 @@ struct tr_handshake
     uint8_t               peer_id[PEER_ID_LEN];
     handshakeDoneCB       doneCB;
     void *                doneUserData;
+    tr_timer *            timeout;
 };
 
 /**
@@ -227,7 +232,7 @@ buildHandshakeMessage( tr_handshake * handshake,
 }
 
 static int tr_handshakeDone( tr_handshake * handshake,
-                             int            isConnected );
+                             tr_bool        isConnected );
 
 enum
 {
@@ -313,7 +318,7 @@ sendYa( tr_handshake * handshake )
 {
     int               len;
     const uint8_t *   public_key;
-    struct evbuffer * outbuf = evbuffer_new( );
+    struct evbuffer * outbuf = tr_getBuffer( );
     uint8_t           pad_a[PadA_MAXLEN];
 
     /* add our public key (Ya) */
@@ -332,7 +337,7 @@ sendYa( tr_handshake * handshake )
     tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
 
     /* cleanup */
-    evbuffer_free( outbuf );
+    tr_releaseBuffer( outbuf );
 }
 
 static uint32_t
@@ -425,7 +430,7 @@ readYb( tr_handshake *    handshake,
 
     /* now send these: HASH('req1', S), HASH('req2', SKEY) xor HASH('req3', S),
      * ENCRYPT(VC, crypto_provide, len(PadC), PadC, len(IA)), ENCRYPT(IA) */
-    outbuf = evbuffer_new( );
+    outbuf = tr_getBuffer( );
 
     /* HASH('req1', S) */
     {
@@ -455,6 +460,7 @@ readYb( tr_handshake *    handshake,
     {
         uint8_t vc[VC_LENGTH] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 
+        tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
         tr_cryptoEncryptInit( handshake->crypto );
         tr_peerIoSetEncryption( handshake->io, PEER_ENCRYPTION_RC4 );
 
@@ -482,7 +488,7 @@ readYb( tr_handshake *    handshake,
     tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
 
     /* cleanup */
-    evbuffer_free( outbuf );
+    tr_releaseBuffer( outbuf );
     return READ_LATER;
 }
 
@@ -842,19 +848,17 @@ readCryptoProvide( tr_handshake *    handshake,
     tr_sha1( req3, "req3", 4, handshake->mySecret, KEY_LEN, NULL );
     for( i = 0; i < SHA_DIGEST_LENGTH; ++i )
         obfuscatedTorrentHash[i] = req2[i] ^ req3[i];
-    if( ( tor =
-             tr_torrentFindFromObfuscatedHash( handshake->session,
-                                               obfuscatedTorrentHash ) ) )
+    if(( tor = tr_torrentFindFromObfuscatedHash( handshake->session, obfuscatedTorrentHash )))
     {
+        const tr_bool clientIsSeed = tr_torrentIsSeed( tor );
+        const tr_bool peerIsSeed = tr_peerMgrPeerIsSeed( tor, tr_peerIoGetAddress( handshake->io, NULL ) );
         dbgmsg( handshake, "got INCOMING connection's encrypted handshake for torrent [%s]",
                 tor->info.name );
         tr_peerIoSetTorrentHash( handshake->io, tor->info.hash );
-        if( !tr_torrentAllowsPex( tor )
-          && tr_peerMgrPeerIsSeed( handshake->session->peerMgr,
-                                  tor->info.hash,
-                                  tr_peerIoGetAddress( handshake->io, NULL ) ) )
+
+        if( clientIsSeed && peerIsSeed )
         {
-            dbgmsg( handshake, "a peer has tried to reconnect to us!" );
+            dbgmsg( handshake, "another seed tried to reconnect to us!" );
             return tr_handshakeDone( handshake, FALSE );
         }
     }
@@ -918,7 +922,7 @@ readIA( tr_handshake *    handshake,
     **/
 
     tr_cryptoEncryptInit( handshake->crypto );
-    outbuf = evbuffer_new( );
+    outbuf = tr_getBuffer( );
 
     dbgmsg( handshake, "sending vc" );
     /* send VC */
@@ -938,7 +942,7 @@ readIA( tr_handshake *    handshake,
     else
     {
         dbgmsg( handshake, "peer didn't offer an encryption mode we like." );
-        evbuffer_free( outbuf );
+        tr_releaseBuffer( outbuf );
         return tr_handshakeDone( handshake, FALSE );
     }
 
@@ -953,7 +957,10 @@ readIA( tr_handshake *    handshake,
 
     /* maybe de-encrypt our connection */
     if( crypto_select == CRYPTO_PROVIDE_PLAINTEXT )
+    {
+        tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
         tr_peerIoSetEncryption( handshake->io, PEER_ENCRYPTION_NONE );
+    }
 
     dbgmsg( handshake, "sending handshake" );
     /* send our handshake */
@@ -967,7 +974,7 @@ readIA( tr_handshake *    handshake,
 
     /* send it out */
     tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
-    evbuffer_free( outbuf );
+    tr_releaseBuffer( outbuf );
 
     /* now await the handshake */
     setState( handshake, AWAITING_PAYLOAD_STREAM );
@@ -1075,7 +1082,7 @@ canRead( struct tr_peerIo * io, void * arg, size_t * piece )
 
 static int
 fireDoneFunc( tr_handshake * handshake,
-              int            isConnected )
+              tr_bool        isConnected )
 {
     const uint8_t * peer_id = isConnected && handshake->havePeerID
                               ? handshake->peer_id
@@ -1090,18 +1097,20 @@ fireDoneFunc( tr_handshake * handshake,
     return success;
 }
 
-void
+static void
 tr_handshakeFree( tr_handshake * handshake )
 {
     if( handshake->io )
-        tr_peerIoFree( handshake->io );
+        tr_peerIoUnref( handshake->io ); /* balanced by the ref in tr_handshakeNew */
+
+    tr_timerFree( &handshake->timeout );
 
     tr_free( handshake );
 }
 
 static int
 tr_handshakeDone( tr_handshake * handshake,
-                  int            isOK )
+                  tr_bool        isOK )
 {
     tr_bool success;
 
@@ -1110,6 +1119,8 @@ tr_handshakeDone( tr_handshake * handshake,
 
     success = fireDoneFunc( handshake, isOK );
 
+    tr_handshakeFree( handshake );    
+
     return success ? READ_LATER : READ_ERR;
 }
 
@@ -1155,6 +1166,13 @@ gotError( tr_peerIo  * io UNUSED,
 ***
 **/
 
+static int
+handshakeTimeout( void * handshake )
+{
+    tr_handshakeAbort( handshake );
+    return FALSE;
+}
+
 tr_handshake*
 tr_handshakeNew( tr_peerIo *        io,
                  tr_encryption_mode encryptionMode,
@@ -1170,8 +1188,11 @@ tr_handshakeNew( tr_peerIo *        io,
     handshake->doneCB = doneCB;
     handshake->doneUserData = doneUserData;
     handshake->session = tr_peerIoGetSession( io );
+    handshake->timeout = tr_timerNew( handshake->session, handshakeTimeout, handshake, HANDSHAKE_TIMEOUT_MSEC );
 
+    tr_peerIoRef( io ); /* balanced by the unref in tr_handshakeFree */
     tr_peerIoSetIOFuncs( handshake->io, canRead, NULL, gotError, handshake );
+    tr_peerIoSetEncryption( io, PEER_ENCRYPTION_NONE );
 
     if( tr_peerIoIsIncoming( handshake->io ) )
         setReadState( handshake, AWAITING_HANDSHAKE );
