@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: transmission.h 7455 2008-12-22 00:51:14Z charles $
+ * $Id: transmission.h 7722 2009-01-16 17:31:10Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -45,9 +45,19 @@ extern "C" {
 #ifndef PRIu64
  #define PRIu64 "llu"
 #endif
+#ifndef PRIu32
+ #define PRIu32 "lu"
+#endif
 #include <time.h> /* time_t */
 
+#if defined( WIN32 ) && defined(_MSC_VER)
+ #define TR_INLINE __inline
+#else
+ #define TR_INLINE inline
+#endif
+
 #define SHA_DIGEST_LENGTH 20
+#define TR_INET6_ADDRSTRLEN 46
 
 typedef uint32_t tr_file_index_t;
 typedef uint32_t tr_piece_index_t;
@@ -55,37 +65,60 @@ typedef uint64_t tr_block_index_t;
 typedef uint16_t tr_port;
 typedef uint8_t tr_bool;
 
-
 /**
  * @brief returns Transmission's default configuration file directory.
  *
  * The default configuration directory is determined this way:
  * 1. If the TRANSMISSION_HOME environmental variable is set, its value is used.
- * 2. On Darwin, "${HOME}/Library/Application Support/Transmission" is used.
- * 3. On Windows, "${CSIDL_APPDATA}/Transmission" is used.
- * 4. If XDG_CONFIG_HOME is set, "${XDG_CONFIG_HOME}/transmission" is used.
- * 5. ${HOME}/.config/transmission" is used as a last resort.
+ * 2. On Darwin, "${HOME}/Library/Application Support/${appname}" is used.
+ * 3. On Windows, "${CSIDL_APPDATA}/${appname}" is used.
+ * 4. If XDG_CONFIG_HOME is set, "${XDG_CONFIG_HOME}/${appname}" is used.
+ * 5. ${HOME}/.config/${appname}" is used as a last resort.
  */
-const char* tr_getDefaultConfigDir( void );
+const char* tr_getDefaultConfigDir( const char * appname );
+
+/**
+ * @brief returns Transmisson's default download directory.
+ *
+ * The default download directory is determined this way:
+ * 1. If the HOME environmental variable is set, "${HOME}/Downloads" is used.
+ * 2. On Windows, "${CSIDL_MYDOCUMENTS}/Downloads" is used.
+ * 3. Otherwise, getpwuid(getuid())->pw_dir + "/Downloads" is used.
+ */
+const char* tr_getDefaultDownloadDir( void );
 
 typedef struct tr_ctor tr_ctor;
-typedef struct tr_handle tr_handle;
 typedef struct tr_info tr_info;
 typedef struct tr_torrent tr_torrent;
-typedef tr_handle tr_session;
+typedef struct tr_session tr_session;
 
 
 /**
  * @addtogroup tr_session Session
  *
- * A libtransmission session is created by calling either tr_sessionInitFull()
- * or tr_sessionInit().  libtransmission creates a thread for itself so that
- * it can operate independently of the caller's event loop.  The session will
- * continue until tr_sessionClose() is called.
+ * A libtransmission session is created by calling tr_sessionInit().
+ * libtransmission creates a thread for itself so that it can operate
+ * independently of the caller's event loop.  The session will continue
+ * until tr_sessionClose() is called.
  *
  * @{
  */
 
+typedef enum
+{
+    TR_PREALLOCATE_NONE   = 0,
+    TR_PREALLOCATE_SPARSE = 1,
+    TR_PREALLOCATE_FULL   = 2
+}
+tr_preallocation_mode;
+
+static TR_INLINE tr_bool tr_isPreallocationMode( tr_preallocation_mode m  )
+{
+    return ( m == TR_PREALLOCATE_NONE )
+        || ( m == TR_PREALLOCATE_SPARSE )
+        || ( m == TR_PREALLOCATE_FULL );
+}
+
 typedef enum
 {
     TR_PROXY_HTTP,
@@ -94,57 +127,6 @@ typedef enum
 }
 tr_proxy_type;
 
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_CONFIG_DIR               tr_getDefaultConfigDir( )
-/** @see tr_sessionInitFull */
-#ifdef TR_EMBEDDED
- #define TR_DEFAULT_ENCRYPTION              TR_CLEAR_PREFERRED
-#else
- #define TR_DEFAULT_ENCRYPTION              TR_ENCRYPTION_PREFERRED
-#endif
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PEX_ENABLED              1
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PORT_FORWARDING_ENABLED  0
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PORT                     51413
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PORT_STR                 "51413"
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_LAZY_BITFIELD_ENABLED    1
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_GLOBAL_PEER_LIMIT        200
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PEER_SOCKET_TOS          8
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PEER_SOCKET_TOS_STR      "8"
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_BLOCKLIST_ENABLED        0
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_RPC_ENABLED              0
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_RPC_PORT                 9091
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_RPC_PORT_STR             "9091"
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_RPC_WHITELIST            "127.0.0.1"
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_RPC_WHITELIST_ENABLED    0
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PROXY_ENABLED            0
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PROXY                    NULL
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PROXY_PORT               80
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PROXY_TYPE               TR_PROXY_HTTP
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PROXY_AUTH_ENABLED       0
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PROXY_USERNAME           NULL
-/** @see tr_sessionInitFull */
-#define TR_DEFAULT_PROXY_PASSWORD           NULL
-
 typedef enum
 {
     TR_CLEAR_PREFERRED,
@@ -153,143 +135,157 @@ typedef enum
 }
 tr_encryption_mode;
 
+static TR_INLINE tr_bool tr_isEncryptionMode( tr_encryption_mode m )
+{
+    return ( m == TR_CLEAR_PREFERRED )
+        || ( m == TR_ENCRYPTION_PREFERRED )
+        || ( m == TR_ENCRYPTION_REQUIRED );
+}
+
+
+#define TR_DEFAULT_OPEN_FILE_LIMIT_STR "32"
+#define TR_DEFAULT_RPC_WHITELIST "127.0.0.1"
+#define TR_DEFAULT_RPC_PORT_STR "9091"
+#define TR_DEFAULT_PEER_PORT_STR "51413"
+#define TR_DEFAULT_PEER_SOCKET_TOS_STR "8"
+#define TR_DEFAULT_PEER_LIMIT_GLOBAL_STR "240"
+#define TR_DEFAULT_PEER_LIMIT_TORRENT_STR "60"
+
+#define TR_PREFS_KEY_BLOCKLIST_ENABLED          "blocklist-enabled"
+#define TR_PREFS_KEY_DOWNLOAD_DIR               "download-dir"
+#define TR_PREFS_KEY_DSPEED                     "download-limit"
+#define TR_PREFS_KEY_DSPEED_ENABLED             "download-limit-enabled"
+#define TR_PREFS_KEY_ENCRYPTION                 "encryption"
+#define TR_PREFS_KEY_LAZY_BITFIELD              "lazy-bitfield-enabled"
+#define TR_PREFS_KEY_MSGLEVEL                   "message-level"
+#define TR_PREFS_KEY_OPEN_FILE_LIMIT            "open-file-limit"
+#define TR_PREFS_KEY_PEER_LIMIT_GLOBAL          "peer-limit-global"
+#define TR_PREFS_KEY_PEER_LIMIT_TORRENT         "peer-limit-per-torrent"
+#define TR_PREFS_KEY_PEER_PORT                  "peer-port"
+#define TR_PREFS_KEY_PEER_PORT_RANDOM_ENABLED   "peer-port-random-enabled"
+#define TR_PREFS_KEY_PEER_PORT_RANDOM_LOW       "peer-port-random-low"
+#define TR_PREFS_KEY_PEER_PORT_RANDOM_HIGH      "peer-port-random-high"
+#define TR_PREFS_KEY_PEER_SOCKET_TOS            "peer-socket-tos"
+#define TR_PREFS_KEY_PEX_ENABLED                "pex-enabled"
+#define TR_PREFS_KEY_PORT_FORWARDING            "port-forwarding-enabled"
+#define TR_PREFS_KEY_PROXY_AUTH_ENABLED         "proxy-auth-enabled"
+#define TR_PREFS_KEY_PREALLOCATION              "preallocation"
+#define TR_PREFS_KEY_PROXY_ENABLED              "proxy-enabled"
+#define TR_PREFS_KEY_PROXY_PASSWORD             "proxy-auth-password"
+#define TR_PREFS_KEY_PROXY_PORT                 "proxy-port"
+#define TR_PREFS_KEY_PROXY                      "proxy"
+#define TR_PREFS_KEY_PROXY_TYPE                 "proxy-type"
+#define TR_PREFS_KEY_PROXY_USERNAME             "proxy-auth-username"
+#define TR_PREFS_KEY_RPC_AUTH_REQUIRED          "rpc-authentication-required"
+#define TR_PREFS_KEY_RPC_ENABLED                "rpc-enabled"
+#define TR_PREFS_KEY_RPC_PASSWORD               "rpc-password"
+#define TR_PREFS_KEY_RPC_PORT                   "rpc-port"
+#define TR_PREFS_KEY_RPC_USERNAME               "rpc-username"
+#define TR_PREFS_KEY_RPC_WHITELIST_ENABLED      "rpc-whitelist-enabled"
+#define TR_PREFS_KEY_RPC_WHITELIST              "rpc-whitelist"
+#define TR_PREFS_KEY_USPEED_ENABLED             "upload-limit-enabled"
+#define TR_PREFS_KEY_USPEED                     "upload-limit"
+#define TR_PREFS_KEY_UPLOAD_SLOTS_PER_TORRENT   "upload-slots-per-torrent"
+
+struct tr_benc;
+
 /**
- * @brief Start a libtransmission session.
- * @return an opaque handle to the new session
+ * Add libtransmission's default settings to the benc dictionary.
  *
- * @param configDir
- *  The config directory where libtransmission config subdirectories
- *  will be found, such as "torrents", "resume", and "blocklists".
- *  #TR_DEFAULT_CONFIG_DIR can be used as a default.
+ * Example:
+ * @code
+ *     tr_benc settings;
+ *     int64_t i;
  *
- * @param downloadDir
- *  The default directory to save added torrents.
- *  This can be changed per-session with
- *  tr_sessionSetDownloadDir() and per-torrent with
- *  tr_ctorSetDownloadDir().
+ *     tr_bencInitDict( &settings, 0 );
+ *     tr_sessionGetDefaultSettings( &settings );
+ *     if( tr_bencDictFindInt( &settings, TR_PREFS_KEY_PEER_PORT, &i ) )
+ *         fprintf( stderr, "the default peer port is %d\n", (int)i );
+ *     tr_bencFree( &settings );
+ * @endcode
  *
- * @param tag
- *  Obsolete.  Only used now for locating legacy fastresume files.
- *  This will be removed at some point in the future.
- *  Valid tags: beos cli daemon gtk macos wx
- *
- * @param isPexEnabled
- *  whether or not PEX is allowed for non-private torrents.
- *  This can be changed per-session with
- *  tr_sessionSetPexEnabled().
- *  #TR_DEFAULT_PEX_ENABLED is the default.
- *
- * @param isPortForwardingEnabled
- *  If true, libtransmission will attempt
- *  to find a local UPnP-enabled or NATPMP-enabled
- *  router and forward a port from there to the local
- *  machine.  This is so remote peers can initiate
- *  connections with us.
- *  #TR_DEFAULT_PORT_FORWARDING_ENABLED is the default.
- *
- * @param publicPort
- *  Port number to open for incoming peer connections.
- *  #TR_DEFAULT_PORT is the default.
- *
- * @param encryptionMode
- *  Must be one of #TR_CLEAR_PREFERRED,
- *  #TR_ENCRYPTION_PREFERRED, or #TR_ENCRYPTION_REQUIRED.
- *
- * @param isUploadLimitEnabled
- *  If true, libtransmission will limit the entire
- *  session's upload speed from "uploadLimit".
- *
- * @param uploadLimit
- *  The speed limit to use for the entire session when
- *  "isUploadLimitEnabled" is true.  Units are KiB/s.
- *
- * @param isDownloadLimitEnabled
- *  If true, libtransmission will limit the entire
- *  session's download speed from "downloadLimit".
- *
- * @param downloadLimit
- *  The speed limit to use for the entire session when
- *  "isDownloadLimitEnabled" is true.  Units are KiB/s.
- *
- * @param peerLimit
- *  The maximum number of peer connections allowed in a session.
- *  #TR_DEFAULT_GLOBAL_PEER_LIMIT can be used as a default.
- *
- * @param messageLevel
- *  Verbosity level of libtransmission's logging mechanism.
- *  Must be one of #TR_MSG_ERR, #TR_MSG_INF, #TR_MSG_DBG.
- *
- * @param isMessageQueueingEnabled
- *  If true, then messages will build up in a queue until
- *  processed by the client application.
- *
- * @param isBlocklistEnabled
- *  If true, then Transmission will not allow peer connections
- *  to the IP addressess specified in the blocklist.
- *
- * @param peerSocketTOS
- *
- * @param rpcIsEnabled
- *  If true, then libtransmission will open an http server port
- *  to listen for incoming RPC requests.
- *
- * @param rpcPort
- *  The port on which to listen for incoming RPC requests
- *
- * @param rpcWhitelist
- *  The list of IP addresses allowed to make RPC connections.
- *  @see tr_sessionSetRPCWhitelist()
- *
- * @see TR_DEFAULT_PEER_SOCKET_TOS
- * @see TR_DEFAULT_BLOCKLIST_ENABLED
- * @see TR_DEFAULT_RPC_ENABLED
- * @see TR_DEFAULT_RPC_PORT
- * @see TR_DEFAULT_RPC_WHITELIST
- * @see TR_DEFAULT_RPC_WHITELIST_ENABLED
- * @see tr_sessionClose()
+ * @param initme pointer to a tr_benc dictionary
+ * @see tr_sessionLoadSettings()
+ * @see tr_sessionInit()
+ * @see tr_getDefaultConfigDir()
  */
-tr_session * tr_sessionInitFull( const char *       configDir,
-                                 const char *       tag,
-                                 const char *       downloadDir,
-                                 int                isPexEnabled,
-                                 int                isPortForwardingEnabled,
-                                 int                publicPort,
-                                 tr_encryption_mode encryptionMode,
-                                 int                useLazyBitfield,
-                                 int                useUploadLimit,
-                                 int                uploadLimit,
-                                 int                useDownloadLimit,
-                                 int                downloadLimit,
-                                 int                peerLimit,
-                                 int                messageLevel,
-                                 int                isMessageQueueingEnabled,
-                                 int                isBlocklistEnabled,
-                                 int                peerSocketTOS,
-                                 int                rpcIsEnabled,
-                                 uint16_t           rpcPort,
-                                 int                rpcWhitelistIsEnabled,
-                                 const char *       rpcWhitelist,
-                                 int                rpcPasswordIsEnabled,
-                                 const char *       rpcUsername,
-                                 const char *       rpcPassword,
-                                 int                proxyIsEnabled,
-                                 const char *       proxy,
-                                 int                proxyPort,
-                                 tr_proxy_type      proxyType,
-                                 int                proxyAuthIsEnabled,
-                                 const char *       proxyUsername,
-                                 const char *       proxyPassword );
+void tr_sessionGetDefaultSettings( struct tr_benc * dictionary );
 
+/**
+ * Add the session's configuration settings to the benc dictionary.
+ *
+ * FIXME: this probably belongs in libtransmissionapp
+ *
+ * @param session
+ * @param dictionary
+ * @see tr_sessionGetDefaultSettings()
+ */
+void tr_sessionGetSettings( tr_session *, struct tr_benc * dictionary );
 
-/** @brief Shorter form of tr_sessionInitFull()
-    @deprecated Use tr_sessionInitFull() instead. */
-tr_session *  tr_sessionInit( const char * configDir,
-                              const char * downloadDir,
-                              const char * tag );
+/**
+ * Load settings from the configuration directory's settings.json file,
+ * using libtransmission's default settings as fallbacks for missing keys.
+ *
+ * FIXME: this belongs in libtransmissionapp
+ *
+ * @param configDir the configuration directory to find settings.json
+ * @param initme pointer to an uninitialized tr_benc
+ * @param appName examples: Transmission, transmission-daemon
+ * @see tr_sessionGetDefaultSettings()
+ * @see tr_sessionInit()
+ * @see tr_sessionSaveSettings()
+ */
+void tr_sessionLoadSettings( struct tr_benc  * dictionary,
+                             const char      * configDir,
+                             const char      * appName );
+
+/**
+ * Add the session's configuration settings to the benc dictionary
+ * and save it to the configuration directory's settings.json file.
+ *
+ * FIXME: this belongs in libtransmissionapp
+ *
+ * @param session
+ * @param dictionary
+ * @see tr_sessionLoadSettings()
+ */
+void tr_sessionSaveSettings( tr_session     * session,
+                             const char     * configDir,
+                             struct tr_benc * dictonary );
+
+/**
+ * Initialize a libtransmission session.
+ *
+ * For example, this will instantiate a session with all the default values:
+ * @code
+ *     tr_benc settings;
+ *     tr_session * session;
+ *     const char * configDir;
+ *
+ *     tr_bencInitDict( &settings, 0 );
+ *     tr_sessionGetDefaultSettings( &settings );
+ *     configDir = tr_getDefaultConfigDir( "Transmission" );
+ *     session = tr_sessionInit( "mac", configDir, true, &settings );
+ *
+ *     tr_bencFree( &settings );
+ * @encode
+ *
+ * @param tag "gtk", "macosx", "daemon", etc... this is only for pre-1.30 resume files
+ * @param configDir where Transmission will look for resume files, blocklists, etc.
+ * @param messageQueueingEnabled if false, messages will be dumped to stderr
+ * @param settings libtransmission settings
+ * @see tr_sessionGetDefaultSettings()
+ * @see tr_sessionLoadSettings()
+ * @see tr_getDefaultConfigDir()
+ */
+tr_session * tr_sessionInit( const char     * tag,
+                             const char     * configDir,
+                             tr_bool          messageQueueingEnabled,
+                             struct tr_benc * settings );
 
 /** @brief End a libtransmission session
-    @see tr_sessionInitFull() */
-void         tr_sessionClose( tr_session * );
+    @see tr_sessionInit() */
+void tr_sessionClose( tr_session * );
 
 /**
  * @brief Return the session's configuration directory
@@ -301,7 +297,7 @@ const char * tr_sessionGetConfigDir( const tr_session * );
 
 /**
  * @brief Set the per-session default download folder for new torrents.
- * @see tr_sessionInitFull()
+ * @see tr_sessionInit()
  * @see tr_sessionGetDownloadDir()
  * @see tr_ctorSetDownloadDir()
  */
@@ -311,7 +307,7 @@ void tr_sessionSetDownloadDir( tr_session  * session,
 /**
  * @brief Get the default download folder for new torrents.
  *
- * This is set by tr_sessionInitFull() or tr_sessionSetDownloadDir(),
+ * This is set by tr_sessionInit() or tr_sessionSetDownloadDir(),
  * and can be overridden on a per-torrent basis by tr_ctorSetDownloadDir().
  */
 const char * tr_sessionGetDownloadDir( const tr_session * session );
@@ -322,27 +318,27 @@ const char * tr_sessionGetDownloadDir( const tr_session * session );
  * @details If true, libtransmission will open a server socket to listen
  * for incoming http RPC requests as described in docs/rpc-spec.txt.
  *
- * This is intially set by tr_sessionInitFull() and can be
+ * This is intially set by tr_sessionInit() and can be
  * queried by tr_sessionIsRPCEnabled().
  */
 void tr_sessionSetRPCEnabled( tr_session  * session,
-                              int           isEnabled );
+                              tr_bool       isEnabled );
 
 /** @brief Get whether or not RPC calls are allowed in this session.
-    @see tr_sessionInitFull()
+    @see tr_sessionInit()
     @see tr_sessionSetRPCEnabled() */
-int  tr_sessionIsRPCEnabled( const tr_session * session );
+tr_bool tr_sessionIsRPCEnabled( const tr_session * session );
 
 /** @brief Specify which port to listen for RPC requests on.
-    @see tr_sessionInitFull()
+    @see tr_sessionInit()
     @see tr_sessionGetRPCPort */
 void tr_sessionSetRPCPort( tr_session  * session,
-                           uint16_t      port );
+                           tr_port       port );
 
 /** @brief Get which port to listen for RPC requests on.
-    @see tr_sessionInitFull()
+    @see tr_sessionInit()
     @see tr_sessionSetRPCPort */
-uint16_t  tr_sessionGetRPCPort( const tr_session * session );
+tr_port tr_sessionGetRPCPort( const tr_session * session );
 
 /**
  * @brief Specify a whitelist for remote RPC access
@@ -357,14 +353,14 @@ void   tr_sessionSetRPCWhitelist( tr_session * session,
 
 /** @brief get the Access Control List for allowing/denying RPC requests.
     @return a comma-separated string of whitelist domains.  tr_free() when done.
-    @see tr_sessionInitFull
+    @see tr_sessionInit
     @see tr_sessionSetRPCWhitelist */
 char* tr_sessionGetRPCWhitelist( const tr_session * );
 
 void  tr_sessionSetRPCWhitelistEnabled( tr_session * session,
-                                        int          isEnabled );
+                                        tr_bool      isEnabled );
 
-int   tr_sessionGetRPCWhitelistEnabled( const tr_session * session );
+tr_bool tr_sessionGetRPCWhitelistEnabled( const tr_session * session );
 
 void  tr_sessionSetRPCPassword( tr_session * session,
                                 const char * password );
@@ -374,16 +370,16 @@ void  tr_sessionSetRPCUsername( tr_session * session,
 
 /** @brief get the password used to restrict RPC requests.
     @return the password string. tr_free() when done.
-    @see tr_sessionInitFull()
+    @see tr_sessionInit()
     @see tr_sessionSetRPCPassword() */
 char* tr_sessionGetRPCPassword( const tr_session * session );
 
 char* tr_sessionGetRPCUsername( const tr_session * session  );
 
 void  tr_sessionSetRPCPasswordEnabled( tr_session * session,
-                                       int          isEnabled );
+                                       tr_bool      isEnabled );
 
-int   tr_sessionIsRPCPasswordEnabled( const tr_session * session );
+tr_bool tr_sessionIsRPCPasswordEnabled( const tr_session * session );
 
 
 typedef enum
@@ -432,13 +428,13 @@ void tr_sessionSetRPCCallback( tr_session   * session,
 ***
 **/
 
-int           tr_sessionIsProxyEnabled( const tr_session * );
+tr_bool       tr_sessionIsProxyEnabled( const tr_session * );
 
-int           tr_sessionIsProxyAuthEnabled( const tr_session * );
+tr_bool       tr_sessionIsProxyAuthEnabled( const tr_session * );
 
 const char*   tr_sessionGetProxy( const tr_session * );
 
-int           tr_sessionGetProxyPort( const tr_session * );
+tr_port       tr_sessionGetProxyPort( const tr_session * );
 
 tr_proxy_type tr_sessionGetProxyType( const tr_session * );
 
@@ -446,32 +442,26 @@ const char*   tr_sessionGetProxyUsername( const tr_session * );
 
 const char*   tr_sessionGetProxyPassword( const tr_session * );
 
-void          tr_sessionSetProxyEnabled(                     tr_session *,
-                                                         int isEnabled );
+void          tr_sessionSetProxyEnabled( tr_session * session,
+                                         tr_bool      isEnabled );
 
-void          tr_sessionSetProxyAuthEnabled(                     tr_session *,
-                                                             int isEnabled );
+void          tr_sessionSetProxyAuthEnabled( tr_session * session,
+                                             tr_bool      isEnabled );
 
-void          tr_sessionSetProxy(
-    tr_session *,
-    const char *
-    proxy );
+void          tr_sessionSetProxy( tr_session * session,
+                                  const char * proxy );
 
-void          tr_sessionSetProxyPort(                     tr_session *,
-                                                      int port );
+void          tr_sessionSetProxyPort( tr_session * session,
+                                      tr_port      port );
 
-void          tr_sessionSetProxyType( tr_session *,
-                                      tr_proxy_type );
+void          tr_sessionSetProxyType( tr_session    * session,
+                                      tr_proxy_type   proxy_type );
 
-void          tr_sessionSetProxyUsername(
-    tr_session *,
-    const char *
-    username );
+void          tr_sessionSetProxyUsername( tr_session * session,
+                                          const char * username );
 
-void          tr_sessionSetProxyPassword(
-    tr_session *,
-    const char *
-    password );
+void          tr_sessionSetProxyPassword( tr_session * session,
+                                          const char * password );
 
 /**
 ***
@@ -504,14 +494,14 @@ void               tr_sessionClearStats( tr_session * session );
  * In public torrents, PEX is enabled by default.
  */
 void               tr_sessionSetPexEnabled( tr_session  * session,
-                                            int           isEnabled );
+                                            tr_bool       isEnabled );
 
-int                tr_sessionIsPexEnabled( const tr_session * session );
+tr_bool            tr_sessionIsPexEnabled( const tr_session * session );
 
 void               tr_sessionSetLazyBitfieldEnabled( tr_session * session,
-                                                     int          enabled );
+                                                     tr_bool       enabled );
 
-int                tr_sessionIsLazyBitfieldEnabled( const tr_session * session );
+tr_bool            tr_sessionIsLazyBitfieldEnabled( const tr_session * session );
 
 tr_encryption_mode tr_sessionGetEncryption( tr_session * session );
 
@@ -524,14 +514,16 @@ void               tr_sessionSetEncryption( tr_session          * session,
 */
 
 void  tr_sessionSetPortForwardingEnabled( tr_session  * session,
-                                          int           enabled );
+                                          tr_bool       enabled );
 
-int   tr_sessionIsPortForwardingEnabled( const tr_session  * session );
+tr_bool tr_sessionIsPortForwardingEnabled( const tr_session  * session );
 
 void  tr_sessionSetPeerPort( tr_session  * session,
-                             int           port);
+                             tr_port       port);
 
-int   tr_sessionGetPeerPort( const tr_session * session );
+tr_port tr_sessionSetPeerPortRandom( tr_session  * session );
+
+tr_port tr_sessionGetPeerPort( const tr_session * session );
 
 typedef enum
 {
@@ -545,7 +537,7 @@ tr_port_forwarding;
 
 tr_port_forwarding tr_sessionGetPortForwarding( const tr_session * session );
 
-int                tr_sessionCountTorrents( const tr_session * session );
+int tr_sessionCountTorrents( const tr_session * session );
 
 typedef enum
 {
@@ -580,6 +572,11 @@ void       tr_sessionSetPeerLimit( tr_session  * session,
 
 uint16_t   tr_sessionGetPeerLimit( const tr_session * session );
 
+void       tr_sessionSetPeerLimitPerTorrent( tr_session  * session,
+                                             uint16_t      maxGlobalPeers );
+
+uint16_t   tr_sessionGetPeerLimitPerTorrent( const tr_session * session );
+
 
 /**
  *  Load all the torrents in tr_getTorrentDir().
@@ -590,6 +587,8 @@ tr_torrent ** tr_sessionLoadTorrents( tr_session  * session,
                                       tr_ctor     * ctor,
                                       int         * setmeCount );
 
+int tr_sessionGetActiveTorrentCount( tr_session * session );
+
 /** @} */
 
 /**
@@ -638,9 +637,9 @@ typedef struct tr_msg_list
 }
 tr_msg_list;
 
-void          tr_setMessageQueuing( int isEnabled );
+void          tr_setMessageQueuing( tr_bool isEnabled );
 
-int           tr_getMessageQueuing( void );
+tr_bool       tr_getMessageQueuing( void );
 
 tr_msg_list * tr_getQueuedMessages( void );
 
@@ -666,17 +665,17 @@ void          tr_freeMessageList( tr_msg_list * freeme );
  *
  * Passing NULL for a filename will clear the blocklist.
  */
-int  tr_blocklistSetContent( tr_session * session,
-                             const char * filename );
+int     tr_blocklistSetContent   ( tr_session       * session,
+                                   const char       * filename );
 
-int  tr_blocklistGetRuleCount( const tr_session * session );
+int     tr_blocklistGetRuleCount ( const tr_session * session );
 
-int  tr_blocklistExists( const tr_session * session );
+tr_bool tr_blocklistExists       ( const tr_session * session );
 
-int  tr_blocklistIsEnabled( const tr_session * session );
+tr_bool tr_blocklistIsEnabled    ( const tr_session * session );
 
-void tr_blocklistSetEnabled( tr_session * session,
-                             int          isEnabled );
+void    tr_blocklistSetEnabled   ( tr_session       * session,
+                                   tr_bool            isEnabled );
 
 
 /** @} */
@@ -725,7 +724,7 @@ tr_ctor*    tr_ctorNew( const tr_session * session );
 void        tr_ctorFree( tr_ctor * ctor );
 
 void        tr_ctorSetDeleteSource( tr_ctor * ctor,
-                                    uint8_t   doDelete );
+                                    tr_bool   doDelete );
 
 int         tr_ctorSetMetainfo( tr_ctor *       ctor,
                                 const uint8_t * metainfo,
@@ -745,16 +744,16 @@ void        tr_ctorSetPeerLimit( tr_ctor *   ctor,
 
 /** Set the download folder for the torrent being added with this ctor.
     @see tr_ctorSetDownloadDir()
-    @see tr_sessionInitFull() */
+    @see tr_sessionInit() */
 void        tr_ctorSetDownloadDir( tr_ctor *    ctor,
                                    tr_ctorMode  mode,
                                    const char * directory );
 
 /** Set whether or not the torrent begins downloading/seeding when created.
     (Default: not paused) */
-void        tr_ctorSetPaused( tr_ctor *   ctor,
-                              tr_ctorMode mode,
-                              uint8_t     isPaused );
+void        tr_ctorSetPaused( tr_ctor      * ctor,
+                              tr_ctorMode    mode,
+                              tr_bool        isPaused );
 
 int         tr_ctorGetPeerLimit( const tr_ctor * ctor,
                                  tr_ctorMode     mode,
@@ -762,7 +761,7 @@ int         tr_ctorGetPeerLimit( const tr_ctor * ctor,
 
 int         tr_ctorGetPaused( const tr_ctor * ctor,
                               tr_ctorMode     mode,
-                              uint8_t *       setmeIsPaused );
+                              tr_bool       * setmeIsPaused );
 
 int         tr_ctorGetDownloadDir( const tr_ctor  * ctor,
                                    tr_ctorMode      mode,
@@ -772,7 +771,7 @@ int         tr_ctorGetMetainfo( const tr_ctor         * ctor,
                                 const struct tr_benc ** setme );
 
 int         tr_ctorGetDeleteSource( const tr_ctor  * ctor,
-                                    uint8_t        * setmeDoDelete );
+                                    tr_bool        * setmeDoDelete );
 
 /* returns NULL if tr_ctorSetMetainfoFromFile() wasn't used */
 const char* tr_ctorGetSourceFile( const tr_ctor * ctor );
@@ -796,6 +795,11 @@ int tr_torrentParse( const tr_session  * session,
                      const tr_ctor     * ctor,
                      tr_info           * setme_info_or_NULL );
 
+/** @brief free a metainfo
+    @see tr_torrentParse */
+void tr_metainfoFree( tr_info * inf );
+
+
 /** Instantiate a single torrent.
     @return 0 on success,
             TR_EINVALID if the torrent couldn't be parsed, or
@@ -828,14 +832,16 @@ void tr_torrentStart( tr_torrent * torrent );
 /** @brief Stop (pause) a torrent */
 void tr_torrentStop( tr_torrent * torrent );
 
-/**
- * @brief Iterate through the torrents.
- *
- * Pass in a NULL pointer to get the first torrent.
- */
-tr_torrent* tr_torrentNext( tr_session  * session,
-                            tr_torrent  * current );
+typedef int tr_fileFunc( const char * filename );
 
+/**
+ * @brief Deletes the torrent's local data.
+ * @param torrent
+ * @param fileFunc Pass in "unlink" to destroy the files or, on platforms with
+ *                 recycle bins, pass in a function that uses it instead.
+ *                 tr_torrentDeleteLocalData() ignores fileFunc's return value.
+ */
+void tr_torrentDeleteLocalData( tr_torrent * torrent,  tr_fileFunc fileFunc );
 
 uint64_t tr_torrentGetBytesLeftToAllocate( const tr_torrent * torrent );
 
@@ -933,7 +939,7 @@ int tr_torrentGetFileDL( const tr_torrent  * torrent,
 void            tr_torrentSetFileDLs( tr_torrent       * torrent,
                                       tr_file_index_t  * files,
                                       tr_file_index_t    fileCount,
-                                      int                do_download );
+                                      tr_bool            do_download );
 
 
 const tr_info * tr_torrentInfo( const tr_torrent * torrent );
@@ -978,9 +984,9 @@ void tr_torrentSetAnnounceList( tr_torrent *            torrent,
 
 typedef enum
 {
-    TR_CP_INCOMPLETE,   /* doesn't have all the desired pieces */
-    TR_CP_DONE,         /* has all the desired pieces, but not all pieces */
-    TR_CP_COMPLETE      /* has every piece */
+    TR_LEECH,           /* doesn't have all the desired pieces */
+    TR_SEED,            /* has the entire torrent */
+    TR_PARTIAL_SEED     /* has the desired pieces, but not the entire torrent */
 }
 tr_completeness;
 
@@ -991,8 +997,8 @@ typedef void ( tr_torrent_completeness_func )( tr_torrent       * torrent,
 /**
  * Register to be notified whenever a torrent's "completeness"
  * changes.  This will be called, for example, when a torrent
- * finishes downloading and changes from TR_CP_INCOMPLETE to
- * either TR_CP_COMPLETE or TR_CP_DONE.
+ * finishes downloading and changes from TR_LEECH to
+ * either TR_SEED or TR_PARTIAL_SEED.
  *
  * func is invoked FROM LIBTRANSMISSION'S THREAD!
  * This means func must be fast (to avoid blocking peers),
@@ -1046,7 +1052,7 @@ typedef struct tr_peer_stat
     uint8_t      from;
     tr_port      port;
 
-    char         addr[16];
+    char         addr[TR_INET6_ADDRSTRLEN];
     char         client[80];
     char         flagStr[32];
 
@@ -1135,8 +1141,8 @@ tr_piece;
 struct tr_info
 {
     /* Flags */
-    unsigned int       isPrivate   : 1;
-    unsigned int       isMultifile : 1;
+    tr_bool            isPrivate;
+    tr_bool            isMultifile;
 
     /* General info */
     uint8_t            hash[SHA_DIGEST_LENGTH];
@@ -1306,16 +1312,22 @@ typedef struct tr_stat
     /** Number of leechers that the tracker says this torrent has */
     int    leechers;
 
+    /** Number of downloaders that the tracker says this torrent has.
+        This is a new key introduced in BEP 21 and may not be supported by some trackers.
+        If the tracker doesn't support this key, the value here will be -1. */
+    int    downloaders;
+
     /** Number of finished downloads that the tracker says torrent has */
     int    timesCompleted;
 
     /** Byte count of all the piece data we'll have downloaded when we're done,
-        whether or not we have it yet. [0...tr_info.totalSize] */
+        whether or not we have it yet.  This may be less than tr_info.totalSize
+        if only some of the torrent's files are wanted.
+        [0...tr_info.totalSize] */
     uint64_t    sizeWhenDone;
 
-    /** Byte count of how much data is left to be downloaded until
-        we're done -- that is, until we've got all the pieces we wanted.
-        [0...tr_info.sizeWhenDone] */
+    /** Byte count of how much data is left to be downloaded until we've got
+        all the pieces that we want.  [0...tr_info.sizeWhenDone] */
     uint64_t    leftUntilDone;
 
     /** Byte count of all the piece data we want and don't have yet,
@@ -1429,14 +1441,13 @@ void tr_torrentSetDoneDate( tr_torrent  * torrent,
                             time_t        doneDate );
 
 /** @brief Sanity checker to test that the direction is TR_UP or TR_DOWN */
-tr_bool tr_isDirection( tr_direction );
+static TR_INLINE tr_bool tr_isDirection( tr_direction d ) { return d==TR_UP || d==TR_DOWN; }
+
+/** @brief Sanity checker to test that a bool is TRUE or FALSE */
+static TR_INLINE tr_bool tr_isBool( tr_bool b ) { return b==1 || b==0; }
 
 /** @} */
 
-#ifdef __TRANSMISSION__
- #include "session.h"
-#endif
-
 #ifdef __cplusplus
 }
 #endif
