@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: DragOverlayWindow.m 7013 2008-11-01 22:08:02Z livings124 $
+ * $Id: DragOverlayWindow.m 7666 2009-01-11 18:04:45Z livings124 $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -25,11 +25,10 @@
 #import "DragOverlayWindow.h"
 #import "DragOverlayView.h"
 #import "NSStringAdditions.h"
-#import "metainfo.h"
 
 @implementation DragOverlayWindow
 
-- (id) initWithLib: (tr_handle *) lib forWindow: (NSWindow *) window
+- (id) initWithLib: (tr_session *) lib forWindow: (NSWindow *) window
 {
     if ((self = ([super initWithContentRect: NSMakeRect(0, 0, 1.0, 1.0) styleMask: NSBorderlessWindowMask
                     backing: NSBackingStoreBuffered defer: NO])))
