@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: StatsWindowController.h 5668 2008-04-22 21:04:01Z livings124 $
+ * $Id: StatsWindowController.h 7666 2009-01-11 18:04:45Z livings124 $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -34,7 +34,7 @@
     NSTimer * fTimer;
 }
 
-+ (StatsWindowController *) statsWindow: (tr_handle *) lib;
++ (StatsWindowController *) statsWindow: (tr_session *) lib;
 
 - (void) resetStats: (id) sender;
 
