@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: Torrent.h 5678 2008-04-24 06:35:21Z livings124 $
+ * $Id: Torrent.h 7666 2009-01-11 18:04:45Z livings124 $
  *
- * Copyright (c) 2006-2008 Transmission authors and contributors
+ * Copyright (c) 2006-2009 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,8 @@
 #import <Cocoa/Cocoa.h>
 #import <transmission.h>
 
+@class FileListNode;
+
 #define INVALID -99
 
 typedef enum
@@ -34,16 +36,16 @@ typedef enum
     TORRENT_FILE_DEFAULT
 } torrentFileState;
 
+#define STAT_TIME_NONE -1
+#define STAT_TIME_NOW -2
+
 @interface Torrent : NSObject
 {
     tr_torrent * fHandle;
     const tr_info * fInfo;
     const tr_stat * fStat;
     
-    int fID;
-    
     BOOL fResumeOnWake;
-    NSDate * fDateAdded, * fDateCompleted, * fDateActivity;
     
     BOOL fUseIncompleteFolder;
     NSString * fDownloadFolder, * fIncompleteFolder;
@@ -58,24 +60,26 @@ typedef enum
     NSString * fNameString, * fHashString;
     
     tr_file_stat * fFileStat;
-    NSArray * fFileList;
+    NSArray * fFileList, * fFlatFileList;
     
-    float * fPreviousFinishedPieces;
-    NSDate * fFinishedPiecesDate;
+    NSIndexSet * fPreviousFinishedIndexes;
+    NSDate * fPreviousFinishedIndexesDate;
     
-    float fRatioLimit;
-    int fRatioSetting;
+    CGFloat fRatioLimit;
+    NSInteger fRatioSetting;
     BOOL fFinishedSeeding, fWaitToStart, fStalled;
     
-    int fOrderValue, fGroupValue;
+    NSInteger fOrderValue, fGroupValue;
+    
+    BOOL fAddedTrackers;
     
     NSDictionary * fQuickPauseDict;
 }
 
 - (id) initWithPath: (NSString *) path location: (NSString *) location deleteTorrentFile: (torrentFileState) torrentDelete
-        lib: (tr_handle *) lib;
-- (id) initWithData: (NSData *) data location: (NSString *) location lib: (tr_handle *) lib;
-- (id) initWithHistory: (NSDictionary *) history lib: (tr_handle *) lib;
+        lib: (tr_session *) lib;
+- (id) initWithTorrentStruct: (tr_torrent *) torrentStruct location: (NSString *) location lib: (tr_session *) lib;
+- (id) initWithHistory: (NSDictionary *) history lib: (tr_session *) lib;
 
 - (NSDictionary *) history;
 
@@ -85,10 +89,10 @@ typedef enum
 - (void) changeDownloadFolder: (NSString *) folder;
 - (NSString *) downloadFolder;
 
-- (void) getAvailability: (int8_t *) tab size: (int) size;
-- (void) getAmountFinished: (float *) tab size: (int) size;
-- (float *) getPreviousAmountFinished;
--(void) setPreviousAmountFinished: (float *) tab;
+- (void) getAvailability: (int8_t *) tab size: (NSInteger) size;
+- (void) getAmountFinished: (float *) tab size: (NSInteger) size;
+- (NSIndexSet *) previousFinishedPieces;
+-(void) setPreviousFinishedPieces: (NSIndexSet *) indexes;
 
 - (void) update;
 
@@ -102,18 +106,18 @@ typedef enum
 
 - (void) resetCache;
 
-- (float) ratio;
-- (int) ratioSetting;
-- (void) setRatioSetting: (int) setting;
-- (float) ratioLimit;
-- (void) setRatioLimit: (float) limit;
-- (float) actualStopRatio; //returns INVALID if will not stop
-- (float) progressStopRatio;
+- (CGFloat) ratio;
+- (NSInteger) ratioSetting;
+- (void) setRatioSetting: (NSInteger) setting;
+- (CGFloat) ratioLimit;
+- (void) setRatioLimit: (CGFloat) limit;
+- (CGFloat) actualStopRatio; //returns INVALID if will not stop
+- (CGFloat) progressStopRatio;
 
 - (tr_speedlimit) speedMode: (BOOL) upload;
 - (void) setSpeedMode: (tr_speedlimit) mode upload: (BOOL) upload;
-- (int) speedLimit: (BOOL) upload;
-- (void) setSpeedLimit: (int) limit upload: (BOOL) upload;
+- (NSInteger) speedLimit: (BOOL) upload;
+- (void) setSpeedLimit: (NSInteger) limit upload: (BOOL) upload;
 
 - (void) setMaxPeerConnect: (uint16_t) count;
 - (uint16_t) maxPeerConnect;
@@ -135,28 +139,32 @@ typedef enum
 - (NSImage *) icon;
 
 - (NSString *) name;
-- (BOOL) folder;
+- (BOOL) isFolder;
 - (uint64_t) size;
 - (uint64_t) sizeLeft;
 
 - (NSString *) trackerAddressAnnounce;
 - (NSDate *) lastAnnounceTime;
-- (int) nextAnnounceTime;
+- (NSInteger) nextAnnounceTime;
 - (NSString *) announceResponse;
 
 - (NSString *) trackerAddressScrape;
 - (NSDate *) lastScrapeTime;
-- (int) nextScrapeTime;
+- (NSInteger) nextScrapeTime;
 - (NSString *) scrapeResponse;
 
-- (NSArray *) allTrackers: (BOOL) separators;
+- (NSMutableArray *) allTrackers: (BOOL) separators;
+- (NSArray *) allTrackersFlat;
+- (BOOL) updateAllTrackersForAdd: (NSMutableArray *) trackers;
+- (void) updateAllTrackersForRemove: (NSMutableArray *) trackers;
+- (BOOL) hasAddedTrackers;
 
 - (NSString *) comment;
 - (NSString *) creator;
 - (NSDate *) dateCreated;
 
-- (int) pieceSize;
-- (int) pieceCount;
+- (NSInteger) pieceSize;
+- (NSInteger) pieceCount;
 - (NSString *) hashString;
 - (BOOL) privateTorrent;
 
@@ -166,15 +174,15 @@ typedef enum
 
 - (BOOL) publicTorrent;
 
-- (float) progress;
-- (float) progressDone;
-- (float) progressLeft;
-- (float) checkingProgress;
+- (CGFloat) progress;
+- (CGFloat) progressDone;
+- (CGFloat) progressLeft;
+- (CGFloat) checkingProgress;
 
-- (int) eta;
-- (int) etaRatio;
+- (NSInteger) eta;
+- (NSInteger) etaRatio;
 
-- (float) notAvailableDesired;
+- (CGFloat) notAvailableDesired;
 
 - (BOOL) isActive;
 - (BOOL) isSeeding;
@@ -187,6 +195,9 @@ typedef enum
 
 - (NSArray *) peers;
 
+- (NSUInteger) webSeedCount;
+- (NSArray *) webSeeds;
+
 - (NSString *) progressString;
 - (NSString *) statusString;
 - (NSString *) shortStatusString;
@@ -194,51 +205,52 @@ typedef enum
 
 - (NSString *) stateString;
 
-- (int) seeders;
-- (int) leechers;
-- (int) completedFromTracker;
+- (NSInteger) seeders;
+- (NSInteger) leechers;
+- (NSInteger) completedFromTracker;
 
-- (int) totalPeersConnected;
-- (int) totalPeersTracker;
-- (int) totalPeersIncoming;
-- (int) totalPeersCache;
-- (int) totalPeersPex;
-- (int) totalPeersKnown;
+- (NSInteger) totalPeersConnected;
+- (NSInteger) totalPeersTracker;
+- (NSInteger) totalPeersIncoming;
+- (NSInteger) totalPeersCache;
+- (NSInteger) totalPeersPex;
+- (NSInteger) totalPeersKnown;
 
-- (int) peersSendingToUs;
-- (int) peersGettingFromUs;
+- (NSInteger) peersSendingToUs;
+- (NSInteger) peersGettingFromUs;
 
-- (float) downloadRate;
-- (float) uploadRate;
-- (float) totalRate;
+- (CGFloat) downloadRate;
+- (CGFloat) uploadRate;
+- (CGFloat) totalRate;
 - (uint64_t) haveVerified;
 - (uint64_t) haveTotal;
 - (uint64_t) totalSizeSelected;
 - (uint64_t) downloadedTotal;
 - (uint64_t) uploadedTotal;
 - (uint64_t) failedHash;
-- (float) swarmSpeed;
+- (CGFloat) swarmSpeed;
 
-- (int) orderValue;
-- (void) setOrderValue: (int) orderValue;
+- (NSInteger) orderValue;
+- (void) setOrderValue: (NSInteger) orderValue;
 
-- (int) groupValue;
-- (void) setGroupValue: (int) groupValue;
-- (int) groupOrderValue;
+- (NSInteger) groupValue;
+- (void) setGroupValue: (NSInteger) groupValue;
+- (NSInteger) groupOrderValue;
 - (void) checkGroupValueForRemoval: (NSNotification *) notification;
 
 - (NSArray *) fileList;
-- (int) fileCount;
+- (NSInteger) fileCount;
 - (void) updateFileStat;
+- (NSArray *) flatFileList;
 
 //methods require fileStats to have been updated recently to be accurate
-- (float) fileProgress: (int) index;
-- (BOOL) canChangeDownloadCheckForFile: (int) index;
+- (CGFloat) fileProgress: (FileListNode *) node;
+- (BOOL) canChangeDownloadCheckForFile: (NSInteger) index;
 - (BOOL) canChangeDownloadCheckForFiles: (NSIndexSet *) indexSet;
-- (int) checkForFiles: (NSIndexSet *) indexSet;
-- (void) setFileCheckState: (int) state forIndexes: (NSIndexSet *) indexSet;
-- (void) setFilePriority: (int) priority forIndexes: (NSIndexSet *) indexSet;
-- (BOOL) hasFilePriority: (int) priority forIndexes: (NSIndexSet *) indexSet;
+- (NSInteger) checkForFiles: (NSIndexSet *) indexSet;
+- (void) setFileCheckState: (NSInteger) state forIndexes: (NSIndexSet *) indexSet;
+- (void) setFilePriority: (NSInteger) priority forIndexes: (NSIndexSet *) indexSet;
+- (BOOL) hasFilePriority: (NSInteger) priority forIndexes: (NSIndexSet *) indexSet;
 - (NSSet *) filePrioritiesForIndexes: (NSIndexSet *) indexSet;
 
 - (NSDate *) dateAdded;
@@ -246,12 +258,11 @@ typedef enum
 - (NSDate *) dateActivity;
 - (NSDate *) dateActivityOrAdd;
 
-- (int) stalledMinutes;
+- (NSInteger) stalledMinutes;
 - (BOOL) isStalled;
 
-- (NSNumber *) stateSortKey;
+- (NSInteger) stateSortKey;
 
-- (int) torrentID;
 - (tr_torrent *) torrentStruct;
 
 @end
