@@ -1,31 +1,20 @@
 #! /bin/sh
 #
-# $Id: version.sh 7492 2008-12-24 17:41:10Z livings124 $
+# $Id: version.sh 7886 2009-02-13 16:39:29Z charles $
 
 # convention: -TR MAJOR MINOR MAINT STATUS - (each a single char)
-# STATUS: "X" for prerelease test builds,
+# STATUS: "X" for prerelease beta builds,
 #         "Z" for unsupported trunk builds,
 #         "0" for stable, supported releases
 # these should be the only two lines you need to change
-PEERID_PREFIX="-TR1420-"
-USERAGENT_PREFIX="1.42"
+PEERID_PREFIX="-TR1500-"
+USERAGENT_PREFIX="1.50"
 
-SVN_REVISION=`find ./libtransmission -name "*\.[chmp]" -o -name "*\.cpp" -o -name "*\.po" -o -name "*\.sh" | \
-              xargs grep "\$Id:" | \
-              grep -v third-party | \
-              cut -d"$Id:" -f3 | cut -d" " -f3 | sort -n | tail -n 1`
+SVN_REVISION=`find -E ./libtransmission ./macosx                     \
+                  -regex ".*\.([chmp]|cpp|po|sh)"                    \
+                  -exec grep -oh '\$Id: [^ ]\+ [0-9]\+' {} +         \
+                  | awk '{ if ($3 > max) max = $3} END { print max }'`
 
-#dirty fix to ensure the highest version number is found when running release_builder.sh
-SVN_REVISION_MAC=`find ./macosx -name "*\.[chmp]" -o -name "*\.cpp" -o -name "*\.po" -o -name "*\.sh" | \
-              xargs grep "\$Id:" | \
-              grep -v third-party | \
-              cut -d"$Id:" -f3 | cut -d" " -f3 | sort -n | tail -n 1`
-
-if [ $SVN_REVISION_MAC -gt $SVN_REVISION ]
-then
-	SVN_REVISION=$SVN_REVISION_MAC
-fi
-  
 # Generate files to be included: only overwrite them if changed so make
 # won't rebuild everything unless necessary
 replace_if_differs ()
@@ -48,6 +37,17 @@ cat > libtransmission/version.h.new << EOF
 #define VERSION_STRING_INFOPLIST  $USERAGENT_PREFIX
 #define BUNDLE_VERSION_INFOPLIST  $SVN_REVISION
 EOF
+
+# Add a release definition
+if [ ${PEERID_PREFIX:6:1} = X ]; then
+    line='#define TR_BETA_RELEASE     "BETA"'
+elif [ ${PEERID_PREFIX:6:1} = Z ]; then
+    line='#define TR_NIGHTLY_RELEASE  "NIGHTLY"'
+else
+    line='#define TR_STABLE_RELEASE   "STABLE"'
+fi
+echo $line >> libtransmission/version.h.new
+
 replace_if_differs libtransmission/version.h.new libtransmission/version.h
 
 exit 0
